﻿namespace LiteBlog.Components
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    using LiteBlog.Common;
    using LiteBlog.Common.Entities;
    using LiteBlog.Common.Enums;
    using LiteBlog.Components.Enums;
    using LiteBlog.Components.Contracts;
    using LiteBlog.Common.Components;
    using System.Globalization;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.XmlLayer;
    using LiteBlog.Components.Helpers;


    public class BlogComp : BaseComp<IBlogData>, IBlogComp
    {
        public BlogComp(IBlogData data, ICacheComp cache) : base(data, cache) { }

        public PostComp PostComponent { get; set; }
        public CategoryComp CategoryComponent { get; set; }
        public ArchiveComp ArchiveComponent { get; set; }
        public SettingsComp SettingsComponent { get; set; }
        public CommentComp CommentComponent { get; set; }
        public DocumentComp DocumentComponent { get; set; }

        public List<PostInfo> Get()
        {
            return _data.Get();
        }

        public List<PostInfo> GetPublished()
        {
            return GetBlogInfo().Posts;
        }

        public PostInfo Get(string id)
        {
            return Get().SingleOrDefault(p => p.FileID == id);
        }


        public MethodResult Create(PostInfo postInfo)
        {
            string draftId;
            PostComponent.Create(postInfo.Title, out draftId);
            postInfo.FileID = draftId;
            return MethodResult.CreateSuccess;
        }

        [Resolve("CategoryComponent")]
        public MethodResult Update(string olid, PostInfo pi)
        {
            var postInfo = Get(olid);
            var oldCategories = postInfo.Categories;
            postInfo.Title = pi.Title;
            postInfo.Categories = pi.Categories;
            PostComponent.Update(postInfo);
            if (!DraftPost.IsDraft(pi.FileID))
                UpdateCategories(oldCategories, postInfo.Categories);
            return MethodResult.UpdateSuccess;
        }

        [Resolve("ArchiveComponent")]
        [Resolve("CategoryComponent")]
        [Resolve("CommentComponent")]
        public MethodResult Delete(string id)
        {
            if (!DraftPost.IsDraft(id))
                DeletePostMeta(id);

            PostComponent.Delete(id);
            return MethodResult.DeleteSuccess;
        }

        [Resolve("SettingsComponent")]
        public List<PostInfo> GetDefaultPosts()
        {
            PostCollection postCol = GetBlogInfo();
            return postCol.Posts.Take<PostInfo>(SettingsComponent.Load().PostCount).ToList<PostInfo>();
        }

        public List<PostInfo> GetPostsByAuthor(string author)
        {
            PostCollection postCol = GetBlogInfo();
            List<PostInfo> posts = postCol.Posts;
            return posts.Where(pi => pi.Author == author).ToList();
        }


        public List<PostInfo> GetPostsByCategory(string catID)
        {
            List<PostInfo> list = new List<PostInfo>();
            PostCollection postCol = GetBlogInfo();
            if (postCol.CategoryMap.ContainsKey(catID))
            {
                list = postCol.CategoryMap[catID];
            }
            else
            {
                var posts = postCol.Posts;
                list = posts.Where(p => p.Categories.Select(c => c.CatID).Contains(catID)).ToList();
                postCol.CategoryMap[catID] = list;
                _cache.Put(CacheType.Blog, postCol);
            }

            return list;
        }

        public List<PostInfo> GetPostsByMonth(int year, int month)
        {
            string monthId = ArchiveMonth.GetMonthId(new DateTime(year, month, 1));

            List<PostInfo> list = new List<PostInfo>();
            PostCollection postCol = GetBlogInfo();
            if (postCol.CategoryMap.ContainsKey(monthId))
            {
                list = postCol.CategoryMap[monthId];
            }
            else
            {
                List<PostInfo> posts = postCol.Posts;
                var qry = from postInfo in posts where postInfo.MonthId == monthId select postInfo;
                list = qry.ToList<PostInfo>();
                postCol.CategoryMap[monthId] = list;
                _cache.Put(CacheType.Blog, postCol);
            }

            return list;
        }

        public bool PostExists(string fileId, out string fileId2)
        {
            fileId2 = string.Empty;
            var post = Get().SingleOrDefault(p => p.FileID.ToLower() == fileId.ToLower());
            if (post != null)
                fileId2 = post.FileID;
            return post != null ? true : false;
        }

        internal void ChangeCategory(string olid, string newid)
        {
            var postsToUpdate = GetPostsByCategory(olid);
            foreach (var postToUpdate in postsToUpdate)
            {
                PostComponent.ChangeCategory(postToUpdate.FileID, olid, newid);
            }
            // change all posts, and then change the post metadata
            _data.ChangeCategory(olid, newid);
        }

        internal void ChangeAuthor(string olid, string newid)
        {
            var postsToUpdate = GetPostsByAuthor(olid);
            foreach (var postToUpdate in postsToUpdate)
            {
                PostComponent.ChangeAuthor(postToUpdate.FileID, olid, newid); // to change to include oldid
            }
            // change all posts, and then change the post metadata
            _data.ChangeAuthor(olid, newid); // to change to remove fileid
        }

        private PostCollection GetBlogInfo()
        {
            PostCollection postCol = _cache.Get<PostCollection>(CacheType.Blog);
            if (postCol == null)
            {
                List<PostInfo> posts = _data.GetPublished();
                postCol = new PostCollection();
                postCol.Posts = posts;
                _cache.Put(CacheType.Blog, postCol);
            }
            return postCol;
        }

        [Resolve("ArchiveComponent")]
        [Resolve("CategoryComponent")]
        [Resolve("DocumentComponent")]
        public MethodResult Publish(DraftPost post)
        {
            if (post.Time > DateTimeHelper.Now)
            {
                _data.Schedule(post.DraftID, post.Time);
                return MethodResult.PublishSuccess; // scheduled
            }

            PostComponent.Publish(post);

            if (post.Type == DraftType.Published)
                UpdateCategories(post.OldCategories, post.Categories);
            else
                UpdateCategories(post.Categories);

            if (post.Type != DraftType.Published)
                ArchiveComponent.Increment(post.Time.Month, post.Time.Year);

            if (post.Type == DraftType.Published)
            {
                var originalPost = PostComponent.Load(post.FileID);
                DocumentHelper.DeleteAttachments(DocumentComponent, post, originalPost);
            }

            return MethodResult.PublishSuccess;
        }

        [Resolve("ArchiveComponent")]
        [Resolve("CategoryComponent")]
        public MethodResult PublishScheduled()
        {
            List<PostInfo> drafts = _data.GetScheduled();
            foreach (PostInfo postInfo in drafts)
            {
                if (postInfo.Time < DateTimeHelper.Now)
                {
                    DraftPost post = (DraftPost)PostComponent.Load(postInfo.FileID);
                    Publish(post);
                }
            }
            return MethodResult.PublishSuccess;
        }

        [Resolve("ArchiveComponent")]
        [Resolve("CategoryComponent")]
        [Resolve("CommentComponent")]
        public MethodResult Unpublish(string fileId, out string draftId)
        {
            DeletePostMeta(fileId);
            draftId = string.Empty;
            if (!PostComponent.CheckIfDraftExistsForPublishedPost(fileId))
                draftId = PostComponent.Unpublish(fileId);
            return MethodResult.UnpublishSuccess;
        }

        private void UpdateCategories(List<Category> oldCategories, List<Category> newCategories)
        {
            if (!oldCategories.SequenceEqual(newCategories, Category.Comparer))
            {
                foreach (var oldCategory in oldCategories)
                {
                    if (!newCategories.Contains(oldCategory, Category.Comparer))
                    {
                        CategoryComponent.Decrement(oldCategory.CatID);
                    }
                }

                foreach (var newCategory in newCategories)
                {
                    if (!oldCategories.Contains(newCategory, Category.Comparer))
                    {
                        CategoryComponent.Increment(newCategory.CatID);
                    }
                }
            }
        }

        private void UpdateCategories(List<Category> categories)
        {
            foreach (var category in categories)
            {
                CategoryComponent.Increment(category.CatID);
            }
        }

        private void DeletePostMeta(string fileId)
        {
            var postInfo = Get(fileId);
            if (postInfo.Categories != null && postInfo.Categories.Count > 0)
            {
                foreach (var category in postInfo.Categories)
                {
                    CategoryComponent.Decrement(category.CatID);
                }
            }
            ArchiveComponent.Decrement(postInfo.Time.Month, postInfo.Time.Year);

            var post = PostComponent.Load(fileId);
            foreach (var comment in post.Comments)
                CommentComponent.Delete(comment.ID);
        }

        [Resolve("DocumentComponent")]
        public void DeleteAttachments(string fileId)
        {
            var post = PostComponent.Load(fileId);
            var draftPost = post as DraftPost;
            bool removeCompletely = false;
            if (draftPost != null)
                removeCompletely = (draftPost.Type == DraftType.New) || (draftPost.Type == DraftType.Unpublished);
            else
                removeCompletely = !PostComponent.CheckIfDraftExistsForPublishedPost(fileId);

            // removeCompletely = remove all documents
            if (removeCompletely)
            {
                foreach (var document in post.Documents)
                    DocumentComponent.Remove(document.Id);
            }
            else
            {
                // removePartially
                if (draftPost != null)
                {
                    // we are deleting a draft post, while published post exists.
                    var publishedPost = PostComponent.Load(draftPost.FileID);
                    DocumentHelper.DeleteAttachments(DocumentComponent, publishedPost, draftPost);
                }
                else
                {
                    // we are deleting a published post, while a draft post exists.
                    draftPost = PostComponent.GetDraftForPublishedPost(fileId);
                    DocumentHelper.DeleteAttachments(DocumentComponent, draftPost, post);
                }
            }
        }
    }
}