﻿namespace LiteBlog.Components
{
    using System;
    using System.Globalization;
    using System.Linq;
    using ColorCode;

    using LiteBlog.Common;
    using LiteBlog.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.Components.Contracts;
    using LiteBlog.Components.Enums;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.Components.Helpers;
    using LiteBlog.XmlLayer;
    using System.Collections.Generic;

    public class PostComp : BaseComp<IPostData>, IPostComp
    {
        public PostComp(IPostData data, ICacheComp cache) : base(data, cache) { }

        public CategoryComp CategoryComponent { get; set; }
      
        public Post Load(string fileId)
        {
            var post = _cache.Get<Post>(fileId);
            if (post == null)
            {
                post = _data.Load(fileId);
                post.FormattedTime = post.Time.ToString(ConfigComp.DateFormat, CultureInfo.InvariantCulture);
                post.HighlightedContents = SyntaxHighlighter.Highlight(post.Contents);

                foreach (var c in post.Comments)
                    c.FormattedTime = c.Time.ToString(ConfigComp.DateFormat, CultureInfo.InvariantCulture);

                _cache.PutPost(fileId, post);
            }
            return post;
        }

        public MethodResult Save(Post post)
        {
            _data.Save(post);
            return MethodResult.SaveSuccess;
        }

        internal void ChangeAuthor(string fileID, string oldAuthor, string newAuthor)
        {
            Post post = _data.Load(fileID);
            post.Author = newAuthor;
            _data.Save(post);
        }

        internal void ChangeCategory(string fileID, string oldCatId, string newCatId)
        {
            Post post = _data.Load(fileID);
            var category = post.Categories.SingleOrDefault(c => c.CatID == oldCatId);
            post.Categories.Remove(category);
            if (!string.IsNullOrEmpty(newCatId))
            {
                var newCategory = CategoryComponent.Get(newCatId);
                post.Categories.Add(newCategory);
            }
            _data.Save(post);
        }

        internal void Create(string title, out string draftId)
        {
            var post = _data.Create(title);
            draftId = post.DraftID;
        }

        internal void Update(PostInfo postInfo)
        {
            _data.Update(postInfo);
        }

        internal void Delete(string fileId)
        {
            _data.Delete(fileId);
        }

        internal void Publish(DraftPost post)
        {
            post.FileID = _data.Publish(post);
        }

        internal string Unpublish(string fileId)
        {
            return _data.Unpublish(fileId);
        }

        internal bool CheckIfDraftExistsForPublishedPost(string fileId)
        {
            return _data.GetDraftIdIfDraftExistsForPublishedPost(fileId) != string.Empty;
        }

        internal DraftPost GetDraftForPublishedPost(string fileId)
        {
            DraftPost draftPost = null;
            string draftId = _data.GetDraftIdIfDraftExistsForPublishedPost(fileId);
            if (!string.IsNullOrEmpty(draftId))
                draftPost = LoadDraft(draftId);
            return draftPost;
        }

        public DraftPost EditPublished(string fileId)
        {
            // allow edit published only once!
            string draftId = _data.GetDraftIdIfDraftExistsForPublishedPost(fileId);
            DraftPost draftPost;
            if (string.IsNullOrEmpty(draftId))
                draftPost = _data.EditPublished(fileId);
            else
                draftPost = LoadDraft(draftId);
            return draftPost;
        }

        public bool DeleteAttachment(string fileId, string docid)
        {
            var draftPost = LoadDraft(fileId);
            var document = draftPost.Documents.SingleOrDefault(doc => doc.Id == docid);
            if (document != null)
                draftPost.Documents.Remove(document);
            Save(draftPost);
            return draftPost.Type == DraftType.Published ? false : true;
        }

        private DraftPost LoadDraft(string fileId)
        {
            return (DraftPost)Load(fileId);
        }

        private void SaveDraft(Post post)
        {
            var draft = (DraftPost)post;
            if (draft == null)
                throw new InvalidOperationException();
            Save(draft);
        }
    }
}