﻿namespace LiteBlog.Controllers
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Linq;
    using System.Web;
    using System.Web.Mvc;
    using LiteBlog.Common;

    using LiteBlog.Models;
    using LiteBlog.Controllers.Helpers;
    using LiteBlog.Components;
    using System.Web.Script.Serialization;
    using System.IO;
    using System.Text;
    using LiteBlog.Controllers.Filters;
    using LiteBlog.Common.Entities;
    using LiteBlog.Common.Components;
    using LiteBlog.Components.Contracts;
    using LiteBlog.Controllers.Contracts;

    public class PostController : CompController<PostInfo, string>, IPostController
    {
        private IBlogComp _blogComp = null;
        private ICategoryComp _categoryComp = null;
        private IPostComp _postComp = null;
        private IStatComp _statComp = null;
        private IDocumentComp _documentComp = null;


        public IBlogComp BlogComponent { get { return _blogComp; } }

        public PostController(IBlogComp blogComp, IPostComp postComp, ICategoryComp categoryComp, IStatComp statComp, ISettingsComp settingsComp, IDocumentComp documentComp)
            : base(blogComp, settingsComp)
        {
            _blogComp = blogComp;
            _postComp = postComp;
            _categoryComp = categoryComp;
            _statComp = statComp;
            _documentComp = documentComp;
        }

        public override ActionResult Manage()
        {
            var collection = _component.Get();
            foreach (var post in collection)
                post.FormattedTime = post.Time.ToString(DisplayHelper.ShortDateFormat, null);

            var serializer = new JavaScriptSerializer();
            ViewBag.Json = serializer.Serialize(collection);
            return View("Manage", GetAdminLayout(), null);
        }

        public override ActionResult Create(PostInfo model)
        {
            throw new NotImplementedException();
        }

        public override ActionResult Update(string olid, PostInfo model)
        {
            throw new NotImplementedException();
        }

        public override ActionResult Delete(string id)
        {
            BlogComponent.DeleteAttachments(id);
            return base.Delete(id);
        }

        [HttpGet]
        [Authorize]
        public ActionResult Compose(string id)
        {
            DraftPost post = null;
            if (string.IsNullOrEmpty(id))
            {
                // called from compose
                post = new DraftPost();
                post.Time = DateTimeHelper.Now;
                post.Type = DraftType.New;
            }
            else
            {
                // called from edit
                if (DraftPost.IsDraft(id))
                    post = (DraftPost)_postComp.Load(id);
                else
                    post = _postComp.EditPublished(id);
            }

            post.FormattedTime = post.Time.ToString(DisplayHelper.CalendarDateFormat, null);
            var categories = _categoryComp.Get().Select(c => new { CatID = c.CatID, Name = c.Name });
            var serializer = new JavaScriptSerializer();
            var model = new
            {
                Post = post,
                Categories = categories
            };
            ViewBag.Json = serializer.Serialize(model);
            return View("Compose", GetAdminLayout(), null);
        }

        [PostNotFoundFilter(Order = 1)]
        [CrawlerFilterForPost(Order = 2)]
        [PageExceptionFilter]
        public ActionResult Index(string id)
        {
            // Redirect with the right URL if required.
            // id2 is got from the NotFoundFilter
            string id2 = RouteData.Values["id2"].ToString();
            if (id != id2)
                return RedirectToAction("Index", new { id = id2 });

            var post = _postComp.Load(id);
            ViewBag.Title = post.Title;

            DisplayHelper.SetDisplayTime(post);

            var ser = new JavaScriptSerializer();
            ViewBag.Json = ser.Serialize(post);
            RouteData.Values["post"] = post;
            _statComp.IncrementPosts(id);
            return View("Index", GetSiteLayout(), null);
        }

        [Authorize]
        [AjaxExceptionFilter]
        [HttpGet]
        public ActionResult Preview(string id)
        {
            var post = _postComp.Load(id);
            post.FormattedTime = post.Time.ToString(DisplayHelper.ShortDateFormat, null);
            return View(post);
        }

        [HttpPost]
        [ValidateInput(false)]
        [ValidateFilter]
        [Authorize]
        public ActionResult Publish(DraftPost post)
        {
            UpdatePost(post);
            _postComp.Save(post);
            var result = _blogComp.Publish(post);
            var resultModel = GetResultModel(result, post.Title);
            var resultModelWithData = new ResultModelWithData<string>(resultModel, post.FileID);
            return Json(resultModelWithData);
        }

        [HttpGet]
        [Authorize]
        public ActionResult Unpublish(string id, string title)
        {
            string draftId;
            var result = _blogComp.Unpublish(id, out draftId);
            var resultModel = GetResultModel(result, title);
            var resultModelWithData = new ResultModelWithData<string>(resultModel, draftId);
            return Json(resultModelWithData, JsonRequestBehavior.AllowGet);
        }

        [HttpPost]
        [ValidateInput(false)]
        [ValidateFilter]
        [Authorize]
        [AjaxExceptionFilter]
        public ActionResult Save(DraftPost post)
        {
            UpdatePost(post);
            var result = _postComp.Save(post);
            var resultModel = GetResultModel(result, DateTimeHelper.Now.ToString("hh:mm:ss tt"));
            var resultModelWithData = new ResultModelWithData<string>(resultModel, post.Type == DraftType.New ? post.DraftID : string.Empty);
            return Json(resultModelWithData);
        }

        [HttpPost]
        [Authorize]
        [AjaxExceptionFilter]
        public ActionResult Upload(string fileId)
        {
            var absUrls = new List<string>();
            var relUrls = new List<string>();
            if (Request.Files != null && Request.Files.Count > 0)
            {
                for (int idx = 0; idx < Request.Files.Count; idx++)
                {
                    var file = Request.Files[idx];
                    if (file.ContentLength > 0)
                    {
                        var document = _documentComp.GetNew(file.FileName);
                        file.SaveAs(document.PhysicalPath);
                        absUrls.Add(document.VirtualPath);
                    }
                }
            }
            return Json(absUrls);
        }

        [HttpGet]
        [AjaxExceptionFilter]
        [Authorize]
        public ActionResult DeleteAttachment(string id, string fileId)
        {
            bool remove = true;
            // if you attach without saving the file, and then delete attachment, fileId will be empty.
            if(!string.IsNullOrEmpty(fileId))
                remove = _postComp.DeleteAttachment(fileId, id);
            if (remove)
                _documentComp.Remove(id);
           
            return Json(new ResultModel { Result = true }, JsonRequestBehavior.AllowGet);
        }


        private void UpdatePost(DraftPost post)
        {
            if (string.IsNullOrEmpty(post.DraftID))
            {
                var postInfo = post.AsPostInfo();
                _blogComp.Create(postInfo);
                post.DraftID = postInfo.FileID;
            }
            post.Author = User.Identity.Name;
        }

        public ActionResult Widget()
        {
            Stat stat = this.ControllerContext.HttpContext.Application["Stat"] as Stat;
            return PartialView(stat.PageVisits.Take(10));
        }

        protected override string[] GetMessageParams(PostInfo model)
        {
            return new string[] { model.Title };
        }
    }
}