﻿namespace LiteBlog.XmlLayer
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;
    using System.Linq;
    using System.Xml.Linq;

    using LiteBlog.Common;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Entities;
    using LiteBlog.Common.Enums;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.XmlLayer.Helpers;

    public class BlogData : IBlogData
    {
        private static string _path;
        private static string _dataPath;
        private DraftData _draftData = null;

        public BlogData(string path)
        {
            _path = path + "Blog.xml";
            _dataPath = path;
            _draftData = new DraftData(_dataPath);
        }

        public static string Path
        {
            get
            {
                return _path;
            }
        }

        public void ChangeAuthor(string oldAuthor, string newAuthor)
        {
            var root = XElement.Load(_path);
            var postsToUpdate = root.Elements("Post").Where(elem => (string)elem.Attribute("Author") == oldAuthor);
            foreach (var postToUpdate in postsToUpdate)
            {
                postToUpdate.SetAttributeValue("Author", newAuthor);
            }
            XmlHelper.Save(root, _path);
        }

        public void ChangeCategory(string oldCatID, string newCatID)
        {
            var root = XElement.Load(_path);
            bool found = false;

            foreach (var postElem in root.Elements("Post"))
            {
                string[] ids = postElem.Attribute("CatID").Value.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                int pos = Array.IndexOf(ids, oldCatID);
                if (pos == -1)
                    continue;
                var idList = ids.ToList();
                found = true;
                if (!string.IsNullOrEmpty(newCatID))
                    idList[pos] = newCatID;
                else
                    idList.RemoveAt(pos);
                postElem.SetAttributeValue("CatID", idList.Aggregate((cur, next) => cur + "," + next));
            }

            if (found)
                XmlHelper.Save(root, _path);
        }

        public void ChangeViews(Dictionary<string, PostInfo> pageVisits)
        {
            var root = XElement.Load(_path);
            foreach (KeyValuePair<string, PostInfo> kvp in pageVisits)
            {
                var postElem = root.Elements("Post").SingleOrDefault(elem => (string)elem.Attribute("FileID")==kvp.Key);
                if (postElem != null)
                {
                    postElem.SetAttributeValue("Views", kvp.Value.Views);
                }
            }
            XmlHelper.Save(root, _path);
        }

        internal void Create(PostInfo postInfo)
        {
            var root = XElement.Load(_path);
            var postElem = root.Elements("Post").SingleOrDefault(elem => elem.Attribute("FileID").Value == postInfo.FileID);

            if (postElem == null)
            {
                string catIds = string.Empty;
                if (postInfo.Categories != null && postInfo.Categories.Count > 0)
                    catIds = postInfo.Categories.Select(c => c.CatID).Aggregate((cur, next) => cur + "," + next);

                postElem = new XElement(
                "Post",
                new XAttribute("Title", postInfo.Title),
                new XAttribute("FileID", postInfo.FileID),
                new XAttribute("Author", postInfo.Author),
                new XAttribute("CatID", catIds),
                new XAttribute("MonthID", postInfo.MonthId),
                new XAttribute("Time", postInfo.Time.ToString(DataContext.DateTimeFormat, CultureInfo.InvariantCulture)),
                new XAttribute("Views", 1));

                root.Add(postElem);
                Sort(root);
                XmlHelper.Save(root, _path);
            }
        }

        internal void Delete(string fileId)
        {
            var root = XElement.Load(_path);
            var postElem = root.Elements("Post").Single(elem => (string)elem.Attribute("FileID") == fileId);
            postElem.Remove();
            XmlHelper.Save(root, _path);
        }

        internal void Update(PostInfo postInfo)
        {
            if (postInfo.Type == PostType.Draft)
            {
                var draftData = new DraftData(_dataPath);
                draftData.Update(postInfo);
                return;
            }

            var root = XElement.Load(_path);
            var postElem = root.Elements("Post").Single(elem => (string)elem.Attribute("FileID") == postInfo.FileID);
            postElem.SetAttributeValue("Title", postInfo.Title);
            string catIds = string.Empty;
            if (postInfo.Categories != null && postInfo.Categories.Count > 0)
                catIds = postInfo.Categories.Select(c => c.CatID).Aggregate((cur, next) => cur + "," + next);
            postElem.SetAttributeValue("CatID", catIds);
            postElem.SetAttributeValue("Author", postInfo.Author);
            var savedTime = (string)postElem.Attribute("Time");
            var updateTime = postInfo.Time.ToString(DataContext.DateTimeFormat, CultureInfo.InvariantCulture);
            if (savedTime != updateTime)
            {
                postElem.SetAttributeValue("Time", updateTime);
                Sort(root);
            }
            XmlHelper.Save(root, _path);
        }

        private void Sort(XElement root)
        {
            var postElems = root.Elements("Post").OrderByDescending(elem => DateTime.ParseExact((string)elem.Attribute("Time"), DataContext.DateTimeFormat, null)).ToList();
            root.RemoveNodes();
            foreach (var postElem in postElems)
                root.Add(postElem);
        }

        internal bool PostExists(string fileId)
        {

            var root = XElement.Load(_path);
            return root.Elements("Post").SingleOrDefault(elem => (string)elem.Attribute("FileID") == fileId) != null;
        }

        public List<PostInfo> GetPublished()
        {
            var postInfos = new List<PostInfo>();
            var root = XElement.Load(_path);

            foreach (var postElem in root.Elements("Post"))
            {
                var postInfo = new PostInfo();
                postInfo.Title = postElem.Attribute("Title").Value;
                postInfo.Author = postElem.Attribute("Author").Value;
                postInfo.FileID = postElem.Attribute("FileID").Value;
                var csvIds = postElem.Attribute("CatID").Value;
                var categoryData = new CategoryData(_dataPath);
                postInfo.Categories = categoryData.GetCategories(csvIds);

                postInfo.Time = DateTime.ParseExact(
                        postElem.Attribute("Time").Value, DataContext.DateTimeFormat, CultureInfo.InvariantCulture);

                postInfo.Views = int.Parse(postElem.Attribute("Views").Value);
                postInfo.Type = PostType.Published;
                postInfo.MonthId = postElem.Attribute("MonthID").Value;
                postInfos.Add(postInfo);
            }
            return postInfos;
        }

        public List<PostInfo> Get()
        {
            var postInfos = GetPublished();
            var draftPostInfos = _draftData.Get();
            postInfos.AddRange(draftPostInfos);
            return postInfos.OrderByDescending(pi => pi.Time).ToList();
        }

        public void Schedule(string fileId, DateTime publishDate)
        {
            var draftData = new DraftData(_dataPath);
            draftData.AddSchedule(fileId, publishDate);
        }

        public List<PostInfo> GetScheduled()
        {
            var draftData = new DraftData(_dataPath);
            return draftData.GetScheduled();
        }
    }
}