﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Web;
using System.Xml.Linq;

using LiteBlog.Common;
using LiteBlog.Common.Components;
using LiteBlog.Common.Entities;
using LiteBlog.XmlLayer.Contracts;
using LiteBlog.XmlLayer.Helpers;

namespace LiteBlog.XmlLayer
{
    public class CommentData : ICommentData
    {
        private static string _path;
        private static string _dataPath;

        public CommentData(string path)
        {
            _path = path + "Comment.xml";
            _dataPath = path;
        }

        public static string Path
        {
            get
            {
                return _path;
            }
        }

        public List<Comment> Get()
        {
            var comments = new List<Comment>();
            var root = XElement.Load(_path);

            foreach (XElement commentElem in root.Elements("Comment"))
            {
                var comment = GetFromElement(commentElem);
                comments.Add(comment);
            }

            return comments;
        }

        public string Create(Comment comment)
        {
            string id = CreateInternal(comment);
            if (comment.IsApproved)
            {
                var postData = new PostData(_dataPath);
                postData.CreateComment(comment);
            }
            return id;
        }

        public void Update(Comment comment)
        {
            UpdateInternal(comment);
            if (comment.IsApproved)
            {
                var postData = new PostData(_dataPath);
                postData.UpdateComment(comment);
            }
        }

        public void Delete(Comment comment)
        {
            DeleteInternal(comment.ID);
            if (comment.IsApproved)
            {
                var postData = new PostData(_dataPath);
                postData.DeleteComment(comment.ID, comment.FileID);
            }
        }

        public void Approve(string id)
        {
            var root = XElement.Load(_path);
            var commentElem = root.Elements("Comment").Single(elem => (string)elem.Attribute("ID") == id);
            if (!(bool)commentElem.Attribute("Approved"))
            {
                commentElem.SetAttributeValue("Approved", true);
                XmlHelper.Save(root, _path);

                var comment = GetFromElement(commentElem);
                var postData = new PostData(_dataPath);
                postData.CreateComment(comment);
            }
        }

        public void Reject(string id)
        {
            DeleteInternal(id);
        }

        internal void Refresh(string newFileId)
        {
            var postData = new PostData(_dataPath);
            var post = postData.Load(newFileId);
            var comments = post.Comments;
            comments = comments.OrderByDescending(c => c.Time).ToList();
            var firstComment = comments.FirstOrDefault();
            if (firstComment == null)
                return;
            var oldFileId = firstComment.FileID;

            var root = XElement.Load(_path);
            root.Elements("Comment").Where(elem => (string)elem.Attribute("FileID") == oldFileId).Remove();
            root.Save(_path);

            foreach (var comment in comments)
            {
                comment.FileID = newFileId;
                comment.ID = CreateInternal(comment);
            }

            post.Comments = comments;
            postData.Save(post);
        }

        private string CreateInternal(Comment comment)
        {
            var root = XElement.Load(_path);
            int id = int.Parse(root.Attribute("TotalComments").Value);
            root.Attribute("TotalComments").Value = (++id).ToString();

            XElement commentElem = new XElement(
                "Comment",
                new XAttribute("ID", id),
                new XAttribute("FileID", comment.FileID),
                new XAttribute("Name", comment.Name),
                new XAttribute("Url", comment.Url ?? string.Empty),
                new XAttribute("Ip", comment.Ip),
                new XAttribute("Time", comment.Time.ToString(DataContext.DateTimeFormat, CultureInfo.InvariantCulture)),
                new XAttribute("IsAuthor", comment.IsAuthor),
                new XAttribute("Approved", comment.IsApproved));
            commentElem.Value = comment.Text;
            root.AddFirst(commentElem);

            XmlHelper.Save(root, _path);
            return id.ToString();
        }

        private Comment Get(string id)
        {
            var root = XElement.Load(_path);
            var commentElem = root.Elements("Comment").Single(elem => (string)elem.Attribute("ID") == id);
            return GetFromElement(commentElem);
        }

        private Comment GetFromElement(XElement commentElem)
        {
            var comment = new Comment();
            comment.ID = commentElem.Attribute("ID").Value;
            comment.FileID = commentElem.Attribute("FileID").Value;
            comment.Name = commentElem.Attribute("Name").Value;
            comment.Ip = commentElem.Attribute("Ip").Value;
            comment.Url = commentElem.Attribute("Url").Value;
            comment.Text = HttpUtility.HtmlDecode(XmlHelper.GetInnerXml(commentElem));
            comment.IsApproved = bool.Parse(commentElem.Attribute("Approved").Value);
            if (commentElem.Attribute("IsAuthor") != null)
            {
                comment.IsAuthor = bool.Parse(commentElem.Attribute("IsAuthor").Value);
            }
            comment.FormattedTime = commentElem.Attribute("Time").Value;
            comment.Time = DateTime.ParseExact(
                commentElem.Attribute("Time").Value,
                DataContext.DateTimeFormat,
                CultureInfo.InvariantCulture);
            return comment;
        }

        private void UpdateInternal(Comment comment)
        {
            var root = XElement.Load(_path);
            var commentElem = root.Elements("Comment").Single(elem => (string)elem.Attribute("ID") == comment.ID);
            commentElem.SetAttributeValue("Name", comment.Name);
            commentElem.SetAttributeValue("Url", comment.Url ?? string.Empty);
            commentElem.SetAttributeValue("IsAuthor", comment.IsAuthor);
            commentElem.Value = comment.Text;
            XmlHelper.Save(root, _path);
        }

        private void DeleteInternal(string id)
        {
            var root = XElement.Load(_path);
            var commentElem = root.Elements("Comment").Single(elem => (string)elem.Attribute("ID") == id);
            commentElem.Remove();
            XmlHelper.Save(root, _path);
        }
    }
}