﻿namespace LiteBlog.XmlLayer
{
    using System;
    using System.Globalization;
    using System.IO;
    using System.Linq;
    using System.Text;
    using System.Web;
    using System.Xml;
    using System.Xml.Linq;

    using LiteBlog.Common;
    using LiteBlog.Common.Components;
    using LiteBlog.Common.Entities;
    using System.Text.RegularExpressions;
    using LiteBlog.XmlLayer.Contracts;
    using LiteBlog.XmlLayer.Helpers;
    using System.Collections.Generic;

    public class PostData : IPostData
    {
        private static string _dataPath = string.Empty;

        public PostData(string path)
        {
            _dataPath = path;
        }

        public void Delete(string fileId)
        {
            string path = this.GetFilePath(fileId);
            if (File.Exists(path))
                File.Delete(path);

            if (DraftPost.IsDraft(fileId))
            {
                var draftData = new DraftData(_dataPath);
                draftData.Delete(fileId);
            }
            else
            {
                var blogData = new BlogData(_dataPath);
                blogData.Delete(fileId);
            }
        }

        public Post Load(string fileId)
        {
            bool isDraft = DraftPost.IsDraft(fileId);
            var post = isDraft ? new DraftPost() : new Post();
            string path = this.GetFilePath(fileId);
            var postElem = XElement.Load(path);

            post.FileID = postElem.Attribute("FileID").Value;
            post.Title = postElem.Attribute("Title").Value;
            post.Author = postElem.Attribute("Author").Value;
            // Fill Categories
            var csvIds = postElem.Attribute("CatID").Value;
            var categoryData = new CategoryData(_dataPath);
            post.Categories = categoryData.GetCategories(csvIds);

            post.Time = DateTime.ParseExact(
                postElem.Attribute("Time").Value, DataContext.DateTimeFormat, CultureInfo.InvariantCulture);

            if (isDraft)
            {
                var draftPost = post as DraftPost;
                draftPost.DraftID = fileId;
                string draftType = postElem.Attribute("Type").Value;
                draftPost.Type = (DraftType)Enum.Parse(typeof(DraftType), draftType, true);

                if (draftPost.Type == DraftType.Published)
                {
                    if (postElem.Attribute("OldCatID") != null)
                    {
                        csvIds = postElem.Attribute("OldCatID").Value;
                        draftPost.OldCategories = categoryData.GetCategories(csvIds);
                    }
                }
            }

            XElement docsElem = postElem.Element("Documents");
            if (docsElem != null)
            {
                foreach (XElement docElem in docsElem.Elements("Document"))
                {
                    Document document = new Document();
                    document.VirtualPath = docElem.Attribute("Path").Value;
                    post.Documents.Add(document);
                }
            }

            string contents = HttpContext.Current.Server.HtmlDecode(XmlHelper.GetInnerXml(postElem.Element("Content")));
            post.Contents = contents;

            XElement commentsElem = postElem.Element("Comments");
            if (commentsElem != null)
            {
                foreach (XElement commentElem in commentsElem.Elements("Comment"))
                {
                    Comment comment = new Comment();
                    comment.FileID = post.FileID;
                    comment.ID = commentElem.Attribute("ID").Value;
                    comment.Name = commentElem.Attribute("Name").Value;
                    comment.Text = HttpContext.Current.Server.HtmlDecode(XmlHelper.GetInnerXml(commentElem));

                    // comment.Text = HttpContext.Current.Server.HtmlDecode(comment.Text);
                    comment.Url = commentElem.Attribute("Url").Value;
                    comment.Time = DateTime.ParseExact(
                        commentElem.Attribute("Time").Value,
                        DataContext.DateTimeFormat,
                        CultureInfo.InvariantCulture);
                    comment.Ip = commentElem.Attribute("Ip").Value;
                    if (commentElem.Attribute("IsAuthor") != null)
                    {
                        comment.IsAuthor = bool.Parse(commentElem.Attribute("IsAuthor").Value);
                    }

                    post.Comments.Add(comment);
                }
            }

            return post;
        }

        public void Save(Post post)
        {
            var draftPost = post as DraftPost;
            string path = this.GetFilePath(draftPost == null ? post.FileID : draftPost.DraftID);

            string template = "<?xml version=\"1.0\" encoding=\"utf-8\"?>";
            template += "<Post Title=\"{0}\" FileID=\"{1}\" Author=\"{2}\" Time=\"{3}\" CatID=\"{4}\" Comments=\"{5}\">";
            template += "<Documents></Documents>";
            template += "<Content></Content>";
            template += "<Comments></Comments></Post>";

            string catIds = string.Empty;
            if (post.Categories != null && post.Categories.Count > 0)
                catIds = post.Categories.Select(c => c.CatID).Aggregate((cur, next) => cur + "," + next);

            template = string.Format(
                template,
                post.Title,
                post.FileID,
                post.Author,
                post.Time.ToString(DataContext.DateTimeFormat, CultureInfo.InvariantCulture),
                catIds,
                post.Comments.Count);


            var postElem = XElement.Parse(template);

            var docsElem = postElem.Element("Documents");
            foreach (Document document in post.Documents)
            {
                XElement docElem = new XElement("Document", new XAttribute("Path", document.VirtualPath));
                docsElem.Add(docElem);
            }

            postElem.Element("Content").Value = post.Contents ?? string.Empty;

            var commentsElem = postElem.Element("Comments");
            commentsElem.RemoveAll();

            foreach (Comment comment in post.Comments)
            {
                XElement commentElem = new XElement(
                    "Comment",
                    new XAttribute("ID", comment.ID),
                    new XAttribute("Name", comment.Name),
                    new XAttribute("Url", comment.Url),
                    new XAttribute("IsAuthor", comment.IsAuthor),
                    new XAttribute(
                        "Time", comment.Time.ToString(DataContext.DateTimeFormat, CultureInfo.InvariantCulture)),
                    new XAttribute("Ip", comment.Ip));

                commentElem.Value = comment.Text;
                commentsElem.Add(commentElem);
            }

            if (draftPost != null)
            {
                postElem.SetAttributeValue("Type", draftPost.Type.ToString());
                if (draftPost.Type == DraftType.Published)
                {
                    catIds = string.Empty;
                    if (draftPost.OldCategories != null && draftPost.OldCategories.Count > 0)
                        catIds = draftPost.OldCategories.Select(c => c.CatID).Aggregate((cur, next) => cur + "," + next);
                    postElem.SetAttributeValue("OldCatID", catIds);
                }
            }

            XmlHelper.Save(postElem, path);
        }


        public static string GetCachePath(string fileID)
        {
            return string.Format("{0}Posts/{1}.xml", _dataPath, fileID);
        }

        private string GetFilePath(string fileID)
        {
            return string.Format("{0}Posts/{1}.xml", _dataPath, fileID);
        }

        public DraftPost Create(string title)
        {
            var post = new DraftPost();
            post.FileID = string.Empty;
            post.DraftID = DraftPost.GetNewDraftID(this.GetUniqueFileID(title), DateTimeHelper.Now);
            post.Title = title;
            post.Type = DraftType.New;


            string template = "<?xml version=\"1.0\" encoding=\"utf-8\"?>";
            template +=
                @"<Post Title=""{0}"" FileID=""{1}"" Author="""" Time="""" CatID="""" Comments=""0"" Type=""New"">";
            template += "<Documents></Documents>";
            template += "<Content></Content>";
            template += "<Comments></Comments></Post>";

            string xml = string.Format(template, title, post.FileID);

            XElement postElem = XElement.Parse(xml);

            string path = this.GetFilePath(post.DraftID);
            XmlWriterSettings settings = new XmlWriterSettings();
            settings.Indent = true;
            XmlWriter writer = XmlWriter.Create(path, settings);
            postElem.Save(writer);
            writer.Close();

            DraftData draft = new DraftData(_dataPath);
            draft.Insert(post.DraftID, string.Empty, title, new List<Category>());
            return post;
        }

        public DraftPost EditPublished(string fileId)
        {
            var post = Load(fileId);
            var draft = new DraftPost(post);
            draft.DraftID = DraftPost.GetNewDraftID(fileId, DateTimeHelper.Now);
            Save(draft);

            var draftData = new DraftData(_dataPath);
            draftData.Insert(draft.DraftID, draft.FileID, draft.Title, draft.Categories);
            return draft;
        }

        public string GetDraftIdIfDraftExistsForPublishedPost(string fileId)
        {
            var draftData = new DraftData(_dataPath);
            return draftData.CheckIfDraftExistsForPublishedPost(fileId);
        }

        public string Unpublish(string fileId)
        {
            // Create a draft copy of the post
            var post = Load(fileId);
            var draftPost = new DraftPost(post);
            draftPost.DraftID = DraftPost.GetNewDraftID(fileId, DateTimeHelper.Now);
            draftPost.Type = DraftType.Unpublished;
            draftPost.Time = DateTimeHelper.Now;
            Save(draftPost);

            // delete the entry from blog register
            var blogData = new BlogData(_dataPath);
            blogData.Delete(fileId);

            // insert entry into draft register
            var draftData = new DraftData(_dataPath);
            draftData.Insert(draftPost.DraftID, string.Empty, draftPost.Title, draftPost.Categories, draftPost.Time);
            return draftPost.DraftID;
        }


        public void Update(PostInfo postInfo)
        {
            var post = Load(postInfo.FileID);
            post.Title = postInfo.Title;
            post.Categories = postInfo.Categories;
            post.Time = postInfo.Time;
            post.Author = postInfo.Author;
            Save(post);

            var blogData = new BlogData(_dataPath);
            blogData.Update(postInfo);
        }


        public string Publish(DraftPost post)
        {
            string filePath = GetFilePath(post.DraftID);
            var root = XElement.Load(filePath);
            root.Attribute("Type").Remove();
            var catAttr = root.Attribute("OldCatID");
            if (catAttr != null)
                catAttr.Remove();

            if (post.Type == DraftType.New)
            {
                post.FileID = GetUniqueFileID(post.Title);
                root.SetAttributeValue("FileID", post.FileID);
            }

            string publishPath = GetFilePath(post.FileID);
            XmlHelper.Save(root, publishPath);

            // Delete old post
            Delete(post.DraftID);

            if (post.Type == DraftType.Unpublished)
            {
                var commentData = new CommentData(_dataPath);
                commentData.Refresh(post.FileID);
            }

            // Add an entry into blogdata
            var postInfo = post.AsPostInfo();
            var blogData = new BlogData(_dataPath);
            if (post.Type != DraftType.Published)
                blogData.Create(postInfo);
            else
            {
                // hack a bit -- modify the entry of the published post
                postInfo.Type = Common.Enums.PostType.Published;
                blogData.Update(postInfo);
            }

            return post.FileID;
        }

        internal void CreateComment(Comment comment)
        {
            string path = this.GetFilePath(comment.FileID);
            var root = XElement.Load(path);
            var commentsElem = root.Element("Comments");
            if (commentsElem != null)
            {
                XElement commentElem = new XElement(
                    "Comment",
                    new XAttribute("ID", comment.ID),
                    new XAttribute("Name", comment.Name),
                    new XAttribute("Url", comment.Url),
                    new XAttribute(
                        "Time", comment.Time.ToString(DataContext.DateTimeFormat, CultureInfo.InvariantCulture)),
                    new XAttribute("Ip", comment.Ip),
                    new XAttribute("IsAuthor", comment.IsAuthor));

                commentElem.Value = comment.Text;
                commentsElem.AddFirst(commentElem);
            }

            int count = int.Parse(root.Attribute("Comments").Value);
            root.SetAttributeValue("Comments", count++);
            XmlHelper.Save(root, path);
        }

        internal void UpdateComment(Comment comment)
        {
            string path = this.GetFilePath(comment.FileID);
            var root = XElement.Load(path);
            var commentElem = root.Element("Comments").Elements("Comment").Single(elem => (string)elem.Attribute("ID") == comment.ID);
            commentElem.SetAttributeValue("Name", comment.Name);
            commentElem.SetAttributeValue("Url", comment.Url ?? string.Empty);
            commentElem.SetAttributeValue("IsAuthor", comment.IsAuthor);
            commentElem.Value = comment.Text;

            XmlHelper.Save(root, path);
        }

        internal void DeleteComment(string id, string fileId)
        {
            string path = this.GetFilePath(fileId);
            var root = XElement.Load(path);

            var commentElem = root.Element("Comments").Elements("Comment").SingleOrDefault(elem => (string)elem.Attribute("ID") == id);
            if (commentElem != null)
                commentElem.Remove();

            int count = int.Parse(root.Attribute("Comments").Value);
            root.SetAttributeValue("Comments", --count);
            XmlHelper.Save(root, path);
        }

        private string GetUniqueFileID(string title)
        {
            string fileID = Regex.Replace(title.Replace(" ", "-"), @"[^\w-]", string.Empty, RegexOptions.None);
            string path = this.GetFilePath(fileID);

            if (File.Exists(path))
            {
                int index = 1;
                string tmpID = string.Format("{0}-{1}", fileID, index);
                path = this.GetFilePath(tmpID);
                while (File.Exists(path))
                {
                    index++;
                    tmpID = string.Format("{0}-{1}", fileID, index);
                    path = this.GetFilePath(tmpID);
                }

                fileID = tmpID;
            }

            return fileID;
        }
    }
}