﻿namespace LiteBlog.XmlProviders
{
    using System;
    using System.Collections.Generic;
    using System.Collections.Specialized;
    using System.Configuration;
    using System.Globalization;
    using System.IO;
    using System.Linq;
    using System.Web.Profile;
    using System.Xml;

    using LiteBlog.Common;
    using LiteBlog.Common.Components;

    public class XmlProfileProvider : ProfileProvider
    {
        private TimeZoneInfo _tzi;
        private static readonly string DateFormat = "MM/dd/yyyy hh:mm tt";
        private XmlDocument doc = new XmlDocument();
        private bool firstLoad = true;
        private bool log = false;
        private string path = string.Empty;
        private string path2 = string.Empty;

        public override string ApplicationName { get; set; }
      
        public override int DeleteInactiveProfiles(
            ProfileAuthenticationOption authenticationOption, DateTime userInactiveSinceDate)
        {
            lock (this)
            {
                ProfileInfoCollection collection = new ProfileInfoCollection();
                XmlNodeList profiles = this.doc.SelectNodes("/Profiles/Profile");
                int inactiveCount = 0;
                List<XmlNode> toRemove = new List<XmlNode>();
                if (profiles != null)
                {
                    foreach (XmlNode profile in profiles)
                    {
                        DateTime actDate = DateTime.MinValue;
                        DateTime.TryParseExact(
                            profile.Attributes["LastActivity"].Value, 
                            DateFormat, 
                            CultureInfo.InvariantCulture, 
                            DateTimeStyles.None, 
                            out actDate);
                        if (actDate < userInactiveSinceDate)
                        {
                            toRemove.Add(profile);
                            inactiveCount++;
                        }
                    }

                    foreach (XmlNode removeNode in toRemove)
                    {
                        this.doc.DocumentElement.RemoveChild(removeNode);
                    }
                }

                this.Save();

                return inactiveCount;
            }
        }

        public override int DeleteProfiles(string[] usernames)
        {
            lock (this)
            {
                int removeIndex = 0;
                foreach (string userName in usernames)
                {
                    string xpath = string.Format(@"/Profiles/Profile[@UserName=""{0}""]", userName);
                    XmlNodeList profiles = this.doc.SelectNodes(xpath);
                    if (profiles != null)
                    {
                        foreach (XmlNode profile in profiles)
                        {
                            this.doc.DocumentElement.RemoveChild(profile);
                            removeIndex++;
                        }
                    }
                }

                if (removeIndex > 0)
                {
                    this.Save();
                }

                return removeIndex;
            }
        }

        public override int DeleteProfiles(ProfileInfoCollection profiles)
        {
            List<string> userNames = new List<string>();
            foreach (ProfileInfo profile in profiles)
            {
                userNames.Add(profile.UserName);
            }

            return this.DeleteProfiles(userNames.Distinct<string>().ToArray<string>());
        }

        public override ProfileInfoCollection FindInactiveProfilesByUserName(
            ProfileAuthenticationOption authenticationOption, 
            string usernameToMatch, 
            DateTime userInactiveSinceDate, 
            int pageIndex, 
            int pageSize, 
            out int totalRecords)
        {
            ProfileInfoCollection collection = new ProfileInfoCollection();
            ProfileInfoCollection tmpColl = this.FindProfilesByUserName(
                authenticationOption, usernameToMatch, 0, int.MaxValue, out totalRecords);
            foreach (ProfileInfo profile in tmpColl)
            {
                if (profile.LastActivityDate < userInactiveSinceDate)
                {
                    collection.Add(profile);
                }
            }

            totalRecords = collection.Count;
            return new ProfileInfoCollection();
        }

        public override ProfileInfoCollection FindProfilesByUserName(
            ProfileAuthenticationOption authenticationOption, 
            string usernameToMatch, 
            int pageIndex, 
            int pageSize, 
            out int totalRecords)
        {
            ProfileInfoCollection collection = new ProfileInfoCollection();
            string xpath = string.Format(@"/Profiles/Profile[@UserName=""{0}""]", usernameToMatch);
            XmlNodeList profiles = this.doc.SelectNodes(xpath);
            if (profiles != null)
            {
                totalRecords = profiles.Count;
                int startIndex = pageIndex * pageSize;
                int endIndex = startIndex + pageSize - 1;

                if (totalRecords < (startIndex + 1))
                {
                    return collection;
                }

                if (endIndex > totalRecords - 1)
                {
                    endIndex = totalRecords - 1;
                }

                for (int curIndex = startIndex; curIndex <= endIndex; curIndex++)
                {
                    XmlNode profile = profiles[curIndex];

                    string userName = profile.Attributes["UserName"].Value;
                    DateTime actDate = DateTime.MinValue;
                    DateTime.TryParseExact(
                        profile.Attributes["LastActivity"].Value, 
                        DateFormat, 
                        CultureInfo.InvariantCulture, 
                        DateTimeStyles.None, 
                        out actDate);
                    DateTime updDate = DateTime.MinValue;
                    DateTime.TryParseExact(
                        profile.Attributes["LastUpdated"].Value, 
                        DateFormat, 
                        CultureInfo.InvariantCulture, 
                        DateTimeStyles.None, 
                        out updDate);

                    ProfileInfo info = new ProfileInfo(userName, true, actDate, updDate, profile.InnerXml.Length);
                    collection.Add(info);
                }
            }

            totalRecords = collection.Count;
            return collection;
        }

        public override ProfileInfoCollection GetAllInactiveProfiles(
            ProfileAuthenticationOption authenticationOption, 
            DateTime userInactiveSinceDate, 
            int pageIndex, 
            int pageSize, 
            out int totalRecords)
        {
            ProfileInfoCollection collection = new ProfileInfoCollection();
            ProfileInfoCollection tmpColl = this.GetAllProfiles(authenticationOption, 0, int.MaxValue, out totalRecords);
            foreach (ProfileInfo profile in tmpColl)
            {
                if (profile.LastActivityDate < userInactiveSinceDate)
                {
                    collection.Add(profile);
                }
            }

            totalRecords = collection.Count;
            return collection;
        }

        /// <summary>
        /// The get all profiles.
        /// </summary>
        /// <param name="authenticationOption">
        /// The authentication option.
        /// </param>
        /// <param name="pageIndex">
        /// The page index.
        /// </param>
        /// <param name="pageSize">
        /// The page size.
        /// </param>
        /// <param name="totalRecords">
        /// The total records.
        /// </param>
        /// <returns>
        /// The System.Web.Profile.ProfileInfoCollection.
        /// </returns>
        public override ProfileInfoCollection GetAllProfiles(
            ProfileAuthenticationOption authenticationOption, int pageIndex, int pageSize, out int totalRecords)
        {
            ProfileInfoCollection collection = new ProfileInfoCollection();
            XmlNodeList profiles = this.doc.SelectNodes("/Profiles/Profile");
            if (profiles != null)
            {
                totalRecords = profiles.Count;
                int startIndex = pageIndex * pageSize;
                int endIndex = startIndex + pageSize - 1;

                if (totalRecords < (startIndex + 1))
                {
                    return collection;
                }

                if (endIndex > totalRecords - 1)
                {
                    endIndex = totalRecords - 1;
                }

                for (int curIndex = startIndex; curIndex <= endIndex; curIndex++)
                {
                    XmlNode profile = profiles[curIndex];

                    string userName = profile.Attributes["UserName"].Value;
                    DateTime actDate = DateTime.MinValue;
                    DateTime.TryParseExact(
                        profile.Attributes["LastActivity"].Value, 
                        DateFormat, 
                        CultureInfo.InvariantCulture, 
                        DateTimeStyles.None, 
                        out actDate);
                    DateTime updDate = DateTime.MinValue;
                    DateTime.TryParseExact(
                        profile.Attributes["LastUpdated"].Value, 
                        DateFormat, 
                        CultureInfo.InvariantCulture, 
                        DateTimeStyles.None, 
                        out updDate);

                    ProfileInfo info = new ProfileInfo(userName, true, actDate, updDate, profile.InnerXml.Length);
                    collection.Add(info);
                }
            }

            totalRecords = collection.Count;
            return collection;
        }

        public override int GetNumberOfInactiveProfiles(
            ProfileAuthenticationOption authenticationOption, DateTime userInactiveSinceDate)
        {
            ProfileInfoCollection collection = new ProfileInfoCollection();
            XmlNodeList profiles = this.doc.SelectNodes("/Profiles/Profile");
            int inactiveCount = 0;
            if (profiles != null)
            {
                foreach (XmlNode profile in profiles)
                {
                    DateTime actDate = DateTime.MinValue;
                    DateTime.TryParseExact(
                        profile.Attributes["LastActivity"].Value, 
                        DateFormat, 
                        CultureInfo.InvariantCulture, 
                        DateTimeStyles.None, 
                        out actDate);
                    if (actDate < userInactiveSinceDate)
                    {
                        inactiveCount++;
                    }
                }
            }

            return inactiveCount;
        }

        public override SettingsPropertyValueCollection GetPropertyValues(
            SettingsContext context, SettingsPropertyCollection collection)
        {
            this.Log("Read");
            lock (this)
            {
                // supports serializing as string, primitive types only
                SettingsPropertyValueCollection values = new SettingsPropertyValueCollection();

                string userName = string.Empty;
                if (context["UserName"] != null)
                {
                    userName = (string)context["UserName"];
                }
                else
                {
                    userName = Guid.NewGuid().ToString();
                }

                this.Log(userName);

                string xpath = string.Format(@"/Profiles/Profile[@UserName=""{0}""]", userName);
                XmlNode profile = this.doc.SelectSingleNode(xpath);
                if (profile == null)
                {
                    profile = this.SetDefaultProfile(userName, collection);
                    this.doc.DocumentElement.AppendChild(profile);
                }

                foreach (SettingsProperty prop in collection)
                {
                    SettingsPropertyValue val = new SettingsPropertyValue(prop);
                    XmlNode propNode = profile.SelectSingleNode(prop.Name);
                    if (propNode == null)
                    {
                        propNode = this.SetDefaultValue(prop);
                        profile.AppendChild(propNode);
                    }

                    switch (prop.PropertyType.ToString())
                    {
                        case "System.String":
                            val.PropertyValue = propNode.InnerText;
                            break;
                        case "System.DateTime":
                            DateTime dt = DateTime.MinValue;
                            DateTime.TryParseExact(
                                propNode.InnerText, 
                                DateFormat, 
                                CultureInfo.InvariantCulture, 
                                DateTimeStyles.None, 
                                out dt);
                            val.PropertyValue = dt;
                            break;
                        case "System.Int32":
                            int number = 0;
                            int.TryParse(propNode.InnerText, out number);
                            val.PropertyValue = number;
                            break;
                        default:
                            double number2 = 0;
                            double.TryParse(propNode.InnerText, out number2);
                            break;
                    }

                    values.Add(val);
                }

                profile.Attributes["LastActivity"].Value = GetLocalTime();
                this.Save();
                this.Log("read complete");
                return values;
            }
        }

        public override void Initialize(string name, NameValueCollection config)
        {
            if (this.firstLoad)
            {
                base.Initialize(name, config);
                this.firstLoad = false;
            }
            else
            {
                string dataPath = config["DataPath"];
                this.path = string.Format("{0}Profile.xml", dataPath);
                this.path2 = string.Format("{0}Log.txt", dataPath);
                this.doc.Load(this.path);
                this._tzi = TimeZoneInfo.FindSystemTimeZoneById(config["Timezone"]);
                this.Log("Initialized");
            }
        }

        public override void SetPropertyValues(SettingsContext context, SettingsPropertyValueCollection collection)
        {
            lock (this)
            {
                this.Log("write");

                string userName = string.Empty;
                if (context["UserName"] != null)
                {
                    userName = (string)context["UserName"];
                }
                else
                {
                    return;
                }

                string xpath = string.Format(@"/Profiles/Profile[@UserName=""{0}""]", userName);
                XmlNode profile = this.doc.SelectSingleNode(xpath);
                if (profile == null)
                {
                    SettingsPropertyCollection propCollection = new SettingsPropertyCollection();
                    foreach (SettingsPropertyValue val in collection)
                    {
                        propCollection.Add(new SettingsProperty(val.Property));
                    }

                    profile = this.SetDefaultProfile(userName, propCollection);
                    this.doc.DocumentElement.AppendChild(profile);
                }

                foreach (SettingsPropertyValue val in collection)
                {
                    XmlNode propElem = profile.SelectSingleNode(val.Name);
                    if (propElem == null)
                    {
                        propElem = this.SetDefaultValue(val.Property);
                    }

                    switch (val.Property.PropertyType.ToString())
                    {
                        case "System.DateTime":
                            propElem.InnerText = ((DateTime)val.PropertyValue).ToString(DateFormat, null);
                            break;
                        default:
                            propElem.InnerText = val.PropertyValue.ToString();
                            break;
                    }
                }

                profile.Attributes["LastActivity"].Value = GetLocalTime();
                profile.Attributes["LastUpdated"].Value = GetLocalTime();
                this.Save();
                this.Log("write complete");
            }
        }

        private DateTime LocalNow
        {
            get
            {
                return TimeZoneInfo.ConvertTime(DateTime.Now, _tzi);
            }
        }

        private string GetLocalTime()
        {
            return LocalNow.ToString(DateFormat, CultureInfo.InvariantCulture);
        }

        private void Log(string text)
        {
            if (this.log)
            {
                using (FileStream fs = File.Open(this.path2, FileMode.OpenOrCreate, FileAccess.ReadWrite))
                {
                    using (StreamWriter sw = new StreamWriter(fs))
                    {
                        sw.WriteLine(text);
                        sw.Flush();
                    }
                }
            }
        }

       private void Save()
        {
            try
            {
                XmlWriterSettings settings = new XmlWriterSettings();
                settings.Indent = true;
                XmlWriter writer = XmlWriter.Create(this.path, settings);
                this.doc.Save(writer);
                writer.Close();
            }
            catch (Exception ex)
            {
                this.Log(ex.Message);
            }
        }

        private XmlNode SetDefaultProfile(string userName, SettingsPropertyCollection collection)
        {
            XmlElement profile = this.doc.CreateElement("Profile");
            XmlAttribute nameAttr = this.doc.CreateAttribute("UserName");
            XmlAttribute actAttr = this.doc.CreateAttribute("LastActivity");
            XmlAttribute updAttr = this.doc.CreateAttribute("LastUpdated");

            nameAttr.Value = userName;
            actAttr.Value = GetLocalTime();
            updAttr.Value = GetLocalTime();

            profile.Attributes.Append(nameAttr);
            profile.Attributes.Append(actAttr);
            profile.Attributes.Append(updAttr);

            foreach (SettingsProperty prop in collection)
            {
                XmlNode propElem = this.SetDefaultValue(prop);
                profile.AppendChild(propElem);
            }

            return profile;
        }

        private XmlNode SetDefaultValue(SettingsProperty prop)
        {
            XmlElement propElem = this.doc.CreateElement(prop.Name);
            switch (prop.PropertyType.ToString())
            {
                case "System.DateTime":
                    propElem.InnerText = DateTime.MinValue.ToString(DateFormat, CultureInfo.InvariantCulture);
                    break;
                case "System.String":
                    propElem.InnerText = string.Empty;
                    break;
                default:
                    propElem.InnerText = "0";
                    break;
            }

            return propElem;
        }
    }
}